__doc__ = """ZGsCounter Product"""

__version__ = '1.02'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent
from OFS.CopySupport import CopySource
from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager

import OFS.SimpleItem
import Acquisition, string
import AccessControl
import DateTime
import OFS.DTMLDocument

manage_addZGsCounterForm = HTMLFile('ZGsCounterAdd', globals())

def manage_addZGsCounter(self, id, REQUEST):
	"""Add a ZGsCounter."""
	if REQUEST is not None:
		ctitle = REQUEST.ctitle
		cuser = REQUEST.cuser
		cpass = REQUEST.cpass
		self._setObject(id, ZGsCounter(id, ctitle, cuser, cpass, REQUEST))
		return self.manage_main(self, REQUEST)

def manage_addZBrowserStat(self, id, REQUEST):
	"""Add a ZBrowserStat."""
	if REQUEST is not None:
		self._setObject(id, ZBrowserStat(id, REQUEST))

def manage_addZViaStat(self, id, REQUEST):
	"""Add a ZViaStat."""
	if REQUEST is not None:
		self._setObject(id, ZViaStat(id, REQUEST))

def manage_addZReferStat(self, id, REQUEST):
	"""Add a ZReferStat."""
	if REQUEST is not None:
		self._setObject(id, ZReferStat(id, REQUEST))

def manage_addZDailyStat(self, id, count):
	"""Add a ZDailyStat."""
	self._setObject(id, ZDailyStat(id, count))

class ZDailyStat(OFS.SimpleItem.Item, Persistent, ObjectManager, Acquisition.Implicit, AccessControl.Role.RoleManager, CopySource):
	"""ZDailyStat Item class."""
	meta_type = 'ZDailyStat'
	_canCopy=1

	def __init__(self, id, count):
		# Initialize a new instance of ZDailyStat Item.
		self.id = id
		self.count = count
		self.day = DateTime.DateTime().strftime('%d')
		self.month = DateTime.DateTime().strftime('%M')
		self.year = DateTime.DateTime().strftime('%Y')
		s = DateTime.DateTime().strftime('%a, %d %b %Y')
		s = '%s %s' % ('Daily Status for ', s)
		self.title = s

class ZBrowserStat(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager, CopySource):
	"""ZBrowserStat Item class."""
	meta_type = 'ZBrowserStat'
	_canCopy=1

	def __init__(self, id, REQUEST):
		# Initialize a new instance of ZBrowserStat Item.
		self.id = id
		if REQUEST.has_key('HTTP_USER_AGENT'):
			self.httpbrowser = REQUEST['HTTP_USER_AGENT']
		else:
			self.httpbrowser = "Unknown"
		if (self.httpbrowser == ""): self.httpbrowser = "Unknown"
		self.count = 1

	def title(self):
		# Get http user agent (browser) as title.
		return self.httpbrowser

class ZViaStat(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager, CopySource):
	"""ZViaStat Item class."""
	meta_type = 'ZViaStat'
	_canCopy=1

	def __init__(self, id, REQUEST):
		# Initialize a new instance of ZViaStat Item.
		self.id = id
		if REQUEST.has_key('HTTP_VIA'):
			self.httpvia = REQUEST['HTTP_VIA']
		else:
			self.httpvia = "Unknown"
		if (self.httpvia == ""): self.httpvia = "Unknown"
		self.count = 1

	def title(self):
		# Get http via as title.
		return self.httpvia

class ZReferStat(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager, CopySource):
	"""ZReferStat Item class."""
	meta_type = 'ZReferStat'
	_canCopy=1

	def __init__(self, id, REQUEST):
		# Initialize a new instance of ZReferStat Item.
		self.id = id
		if REQUEST.has_key('HTTP_REFERER'):
			self.httprefer = REQUEST['HTTP_REFERER']
		else:
			self.httprefer = "unknown"
		if (self.httprefer == ""): self.httprefer = "Direct to Site"
		self.count = 1

	def title(self):
		# Get http referer as title.
		return self.httprefer

class ZGsCounter(Folder, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager, CopySource):
	"""ZGsCounter Folder class."""
	meta_type = 'ZGsCounter'
	_canCopy= 1

	def all_meta_types(self):
		return ({'name':'ZBrowserStat','action': 'manage_addZBrowserStat'},
			{'name':'ZViaStat','action': 'manage_addZViaStat'},
			{'name':'ZReferStat','action': 'manage_addZReferStat'},
			{'name':'ZDailyStat','action': 'manage_addZDailyStat'},
			{'name':'DTML Document','action': 'DTMLDocument.manage_addDTMLDocument'},
			)

	manage_options = (
		{'label': 'Custom Counts', 'action': 'manage_counter'},
		{'label': 'Counter Properties', 'action': 'manage_properties'},
		{'label': 'Statistics', 'action': 'manage_stat'},
		)

	def __init__(self, id, ctitle, cuser, cpass, REQUEST):
		"""Initialize a new instance of ZGsCounter."""
		self.id = id
		self.title = ctitle
		self.cookiename = 'countdate'
		self.cuser = cuser
		self.cpass = cpass
		self.ownereml = ''
		self.lastdate = DateTime.DateTime().strftime('%d%m%Y')
		self.dailycount = 0
		self.count = 0
		self.countsave = 0
		self.usehttpvia = 2
		self.usehttprefer = 2
		self.usebrowser = 2
		self.sendstat = 0
		if ctitle != "":
			iTitle = '%s %s' % (ctitle, ' Statisitics')
		else:
			iTitle = '%s %s' % (id, ' Statisitics')
		iPag = '%s%s%s%s%s%s%s%s%s%s%s%s%s' % ('<dtml-var standard_html_header>\n\n',
			'  <H1>Counter Statistics</H1>\n',
			'  <P>Visitors count: <B><dtml-var "setCount(REQUEST)"></B><BR>\n',
			'  Daily visitors count: <B><dtml-var dailycount></B></P>\n',
			'  <TABLE BORDER="0" CELLPADDING="2" CELLSPACING="0" WIDTH="350" STYLE="border: 1px solid #000000">\n',
			'<dtml-in getVisitorsPerDays size=7>\n<dtml-if sequence-start>\n  <TR BGCOLOR="#F0F0F0">\n',
			'    <TD WIDTH="250" STYLE="border-right: 1px solid #A0A0A0; border-bottom: 1px solid #A0A0A0">&nbsp;<B>Date</B></TD>\n',
			'    <TD WIDTH="100" ALIGN="right" STYLE="border-bottom: 1px solid #A0A0A0"><B>Count</B>&nbsp;</TD>\n  </TR>\n</dtml-if>\n',
			'  <TR<dtml-if sequence-odd> BGCOLOR="#FFFFF0"</dtml-if>>\n<dtml-let dinc=sequence-index>\n',
			'    <TD WIDTH="250" STYLE="border-right: 1px solid #A0A0A0">&nbsp;<dtml-var "(ZopeTime() - _.int(dinc))"></TD>\n',
			'    <TD WIDTH="100" ALIGN="right"><dtml-var sequence-item>&nbsp;</TD>\n',
			'</dtml-let>\n  </TR>\n</dtml-in>\n</TABLE>\n \n',
			'<dtml-var standard_html_footer>')
		self.manage_addDTMLDocument('index_html', iTitle, iPag)

	manage_stat = HTMLFile('ZGsCounterStat', globals())
	manage_properties = HTMLFile('ZGsCounterEdit', globals())
	manage_counter = HTMLFile('ZGsCounterMain', globals())
	manage_workspace = manage_counter
	manage_main = manage_counter

	def counter_saver(self, csave = None):
		# Save or reload general counter values.
		if csave is not None:
			if (csave == "save"):
				self.countsave = self.count
			elif (csave == "load"):
				self.count = self.countsave
			elif (csave == "reset"):
				self.count = 0
				self.dailycount = 0
			        for id, ob in self.objectItems('ZBrowserStat'): ob.count = 0
			        for id, ob in self.objectItems('ZReferStat'): ob.count = 0
			        for id, ob in self.objectItems('ZViaStat'): ob.count = 0
	
	def manage_edit(self, REQUEST = None):
		"""Set the new properties."""
		if REQUEST is not None:
			self.title = REQUEST.ctitle
			self.cuser = REQUEST.cuser
			self.cpass = REQUEST.cpass
			self.ownereml = REQUEST.ceml
			self.sendstat = REQUEST.sendstat
			self.cookiename = REQUEST.cookie
			self.usehttpvia = REQUEST.usehttpvia
			self.usehttprefer = REQUEST.usehttprefer
			self.usebrowser = REQUEST.usebrowser
			self.dailycount = REQUEST.dcount
			if REQUEST.lastd != '0':
				if REQUEST.lastm != '0':
					if REQUEST.lasty != '0':
						self.lastdate = '%s%s%s' % (REQUEST.lastd, REQUEST.lastm, REQUEST.lasty)
			if (REQUEST.csave != "none"): 
	 			self.counter_saver(REQUEST.csave)
				if (REQUEST.csave == "load"):
					return MessageDialog(
						title = 'Loaded',
						message = '<FONT FACE="Arial" SIZE="-1">General counter values of ZGsCounter named <B>&quot;%s&quot;</B> has been<BR><FONT COLOR="#008000"><B>LOADED!</B></FONT></FONT>' % (self.id),
						action = './manage_properties',
						)
				elif (REQUEST.csave == "reset"):
					return MessageDialog(
						title = 'Reseted',
						message = '<FONT FACE="Arial" SIZE="-1">General counter values of ZGsCounter named <B>&quot;%s&quot;</B> has been<BR><FONT COLOR="#A00000"><B>RESETED!</B></FONT></FONT>' % (self.id),
						action = './manage_properties',
						)
				else:
					return MessageDialog(
						title = 'Saved',
						message = '<FONT FACE="Arial" SIZE="-1">General counter values of ZGsCounter named <B>&quot;%s&quot;</B> has been<BR><FONT COLOR="#0000A0"><B>SAVED!</B></FONT></FONT>' % (self.id),
						action = './manage_properties',
						)
			else:
				self.count = int(REQUEST.ccount)
				return MessageDialog(
					title = 'Edited',
					message = '<FONT FACE="Arial" SIZE="-1">Properties of ZGsCounter named <B>&quot;%s&quot;</B> has been changed!</FONT>' % (self.id),
					action = './manage_properties',
					)

	def manage_lastday(self, REQUEST = None):
		# Set the new 'Last Day Counter' properties.
		self.lastdate = DateTime.DateTime()
		if REQUEST is not None:
			self.fromlist = REQUEST.fromlist
			return MessageDialog(
				title = 'Edited',
				message = '<FONT FACE="Arial" SIZE="-1">Last day counter of ZGsCounter named <B>&quot;%s&quot;</B> has been changed!</FONT>' % (self.id),
				action = './manage_properties',
				)

	def setVistCookie(self, REQUEST):
		# Add a counter cookie to the visitor.
		dt = DateTime.DateTime()
		cok = dt.strftime('%d%m%Y%H')
		dt = dt + 1
		expr = dt.strftime('%a, %d %b %Y %T GMT')
		response = REQUEST['RESPONSE']
		response.setCookie(self.cookiename, cok, expires=expr)

	def setCustomCounts(self, REQUEST):
	        # Count a visitor by HTTP_VIA, HTTP_REFERER and HTTP_USER AGENT.
		if (int(self.usehttpvia) != 1):
			if REQUEST.has_key('HTTP_VIA'):
				via = REQUEST['HTTP_VIA']
				if (via == ""):
					via = "Unknown"
				isin = 0
			        for id, ob in self.objectItems('ZViaStat'):
					if (via == ob.httpvia):
						isin = 1
						ob.count = int(ob.count) + 1
						break
				if (isin == 0):
					uid = DateTime.DateTime().strftime('%d%m%Y%H%M%S')
					uid = '%s%s' % ('V', uid)
					manage_addZViaStat(self, uid, REQUEST)

		if (int(self.usehttprefer) != 1):
			if REQUEST.has_key('HTTP_REFERER'):
				ref = REQUEST['HTTP_REFERER']
				if (ref == ""):
					ref = "Direct to Site"
				isin = 0
			        for id, ob in self.objectItems('ZReferStat'):
					if (ref == ob.httprefer):
						isin = 1
						ob.count = int(ob.count) + 1
						break
				if (isin == 0):
					uid = DateTime.DateTime().strftime('%d%m%Y%H%M%S')
					uid = '%s%s' % ('R', uid)
					manage_addZReferStat(self, uid, REQUEST)

		if (int(self.usebrowser) != 1):
			if REQUEST.has_key('HTTP_USER_AGENT'):
				ag = REQUEST['HTTP_USER_AGENT']
				if (ag == ""):
					ag = "Unknown"
				isin = 0
			        for id, ob in self.objectItems('ZBrowserStat'):
					if (ag == ob.httpbrowser):
						isin = 1
						ob.count = int(ob.count) + 1
						break
				if (isin == 0):
					uid = DateTime.DateTime().strftime('%d%m%Y%H%M%S')
					uid = '%s%s' % ('B', uid)
					manage_addZBrowserStat(self, uid, REQUEST)			
		
	def setCount(self, REQUEST = None):
	        # Count a visitor and return the total counting value.
		if REQUEST is not None:
			chkdt = DateTime.DateTime().strftime('%d%m%Y')
			uid = '%s%s' % ('daily', chkdt)
			if not REQUEST.has_key(self.cookiename):
				self.setVistCookie(REQUEST)
				self.setCustomCounts(REQUEST)
				self.count = int(self.count) + 1
				try:
					self.dailycount = int(self.dailycount) + 1
					op = self._getOb(uid)
					op.count = self.dailycount
				except:
					self.dailycount = 1
					manage_addZDailyStat(self, uid, 1)
					op = self._getOb(uid)
					self.lastdate = chkdt
			else:
				dt = DateTime.DateTime()
				cok = dt.strftime('%d%m%Y%H')
				if REQUEST[self.cookiename] != cok:
					self.setVistCookie(REQUEST)
					self.setCustomCounts(REQUEST)
					self.count = int(self.count) + 1
					try:
						self.dailycount = int(self.dailycount) + 1
						op = self._getOb(uid)
						op.count = self.dailycount
					except:
						self.dailycount = 1
						manage_addZDailyStat(self, uid, 1)
						op = self._getOb(uid)
						self.lastdate = chkdt
		return self.count

	def getVisitorsPerDays(self, lastDays = 7):
		# Get visitors per days
		r = []
		i = 0
		while (i < lastDays):
			dt = DateTime.DateTime() - i
			uid = '%s%s' % ('daily', dt.strftime('%d%m%Y'))
			try:
				op = self._getOb(uid)
				if (op.id == uid): 
					r.append(op.count)
				else:
					r.append(0)
			except:
				r.append(0)
			if i < lastDays: 
				dt = DateTime.DateTime() - i
			else:
				r.append(int(self.dailycount))
			i = i + 1
		return r

	def BROWSER_STAT(self):
		# Get browsers statistic.
		return self.objectItems('ZBrowserStat')

	def HTTP_VIA_STAT(self):
		# Get http via statistic.
		return self.objectItems('ZViaStat')

	def HTTP_REFERER_STAT(self):
		# Get http referers statistic.
		return self.objectItems('ZReferStat')

	def DigitCounter(self, digits = 5):
		# Get counter digits.
		c = '%s' % self.count
		d = []
		if digits < len(c):
			i = 0
			while i < len(c):
				d.append(int(c[i]))
				i = i + 1
		else:
			i = 0
			while i < (digits - len(c)):
				d.append(0)
				i = i + 1
			i = 0
			while i < len(c):
				d.append(int(c[i]))
				i = i + 1
			
		return d
			